package com.simonstl.moe;

import java.util.Map;
import java.util.Set;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The AnnotableI interface defines how MOE objects can be annotated with type or other information.</p>
*
* <p>version 0.01 is the initial release.</p>
*
* @version 0.01 24 August 2001
* @author Simon St.Laurent
**/

public interface AnnotableI extends Cloneable{

public void addAll(Map mapOfAnnotations);

/**
* The annotate() method allows developers to add different kinds of annotation information to components.
**/
public void annotate(Object annotationType, Object annotation);

/**
* This method allows developers to retrieve annotations of one particular type.
**/
public Object getAnnotations(Object ofAnnotationType);

/**
* This method allows developers to retrieve all annotations at once.
**/
public Map getAllAnnotations();

/**
* Returns a Set containing the types of all annotations stored in this object
**/
public Set getTypes();

/**
* Indicates whether the provided object is stored as an annotation (of any type)
**/
public boolean hasAnnotation(Object annotation);

/**
* Indicates whether there are any annotations of the given type.
**/
public boolean hasAnnotationOfType(Object annotationType);

public boolean hasAnnotations();

/**
* This method allows developers to remove a single annotation.
**/
public void removeAnnotation(Object ofAnnotationType);

/**
* This method allows developers to remove all annotations at once.
**/
public void removeAllAnnotations();

/**
The clone() method will make a deep copy when its contents implement Cloneable.

Content which is not Cloneable will be copies by reference.  Use this flexibility cautiously.
**/
public Object clone();

}