package com.simonstl.moe;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Collections;


/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License.

The Original Code is available at  http://simonstl.com/projects/fragment/original.

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved.

Contributor(s):
*/

/**
 * <p>The Annotations class implements the {@link com.simonstl.moe.AnnotableI AnnotableI}, though
 * access to this class typically occurs through an implementation of that interface in
 * the {@link com.simonstl.moe.CoreComponent CoreComponent} class.</p>
 * <p>All objects placed in the Annotations should implement Cloneable.</p>
 * <p>version 0.01 is the initial - bogus - release.  Only the deprecated get/setType methods are
 * available.  The rest are placeholders.</p>
 * @version 0.01 24 August 2001
 * @author Simon St.Laurent
 */
public class Annotations implements AnnotableI
{
    protected Map annotations;

    public Annotations()
    {
      annotations = new HashMap();
    }

    /** Returns the set of annotation types. */
    public Set getTypes()
    {
        return annotations.keySet();
    }

    public void addAll(Map map)
    {
        annotations.putAll(map);
    }

    public void annotate(Object annotationType, Object annotation)
    {
        annotations.put(annotationType, annotation);
    }

    public Object getAnnotations(Object ofAnnotationType)
    {
        return annotations.get(ofAnnotationType);
    }

    public Map getAllAnnotations()
    {
        return Collections.unmodifiableMap(annotations);
    }

    public boolean hasAnnotation(Object annotation)
    {
        return annotations.containsValue(annotation);
    }

    public boolean hasAnnotationOfType(Object annotationType)
    {
        return annotations.containsKey(annotationType);
    }

    public boolean hasAnnotations()
    {
        return annotations.isEmpty();
    }

    public void removeAnnotation(Object ofAnnotationType)
    {
       annotations.remove(ofAnnotationType);
    }

    public void removeAllAnnotations()
    {
       annotations.clear();
    }

   public Object clone()
   {
      //TODO - this should probably be a deep clone.
      Annotations other = new Annotations();
      other.addAll(annotations);
      return other;

   }
   
}
