package com.simonstl.moe;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLFilterImpl;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.StringTokenizer;
import com.simonstl.moe.explicit.Characters;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The CoreComponent class implements the bare minimum of functionality needed for objects to work with MOE.</p>
*
* <p>Note that while this class implements the  {@link com.simonstl.moe.AnnotableI AnnotableI} interface, it delegates all of the work to an {@link com.simonstl.moe.Annotations Annotations} object.</p>
* <p>Similarly, all of the processing of the  {@link com.simonstl.moe.NamingI NamingI} interface is delegated to a {@link com.simonstl.moe.Name Name} object.</p>
*
* <p>version 0.01 is the initial release and contains plenty of bogus placeholder functionality.</p>
*
* @version 0.01 7 September 2001
* @author Simon St.Laurent
**/

public abstract class CoreComponent implements CoreComponentI, AnnotableI {

/**
* contains the name information for component (if any)
**/
  protected Name name=null;

/**
* contains the ordered content (typically elements) for the component.
**/
  protected ComponentList content=null;
/**
* contains the unordered content (typically attributes) for the component.
**/
  protected ComponentSet unorderedContent=null;
/**
* indicates whether the component contains all of its child parts.
**/
  protected boolean complete=false;
/**
* an int identifying the base type of this component.
**/
  protected int componentType;
/**
* contains the annotations - often type information - for this component.
**/
  protected AnnotableI annotations=new Annotations();
/**
* identifies the parent of this component.
**/
  protected CoreComponentI parent=null;

/**
* used by listen().
**/

  protected CoreComponent context=this;


public CoreComponent()
{
   annotations = new Annotations();
}

/**
* The hashCode() method uses nsURI+">"+localName to generate an int.  If you need something more precise, you'll need to override this.
**/
public int hashCode(){
  String identifier=this.getNsURI()+">"+this.getLocalName();
  return identifier.hashCode();
}

public void setNsURI(String _nsURI) {
   if (name==null) {name=new Name();}
   name.setNsURI(_nsURI);
}

public String getNsURI() {
   if (name==null) {name=new Name();}

   return name.getNsURI();
}

public void setLocalName(String _localName){
   if (name==null) {name=new Name();}

   name.setLocalName(_localName);
}

public String getLocalName(){
   if (name==null) {name=new Name();}

   return name.getLocalName();
}

public void setPrefix(String _prefix){
   if (name==null) {name=new Name();}

   name.setPrefix(_prefix);
}

public String getPrefix(){
   if (name==null) {name=new Name();}

   return name.getPrefix();
}

/**
* Returns the Qualified Name (QName) for the component.
**/
public String getQName(){
   if (name==null) {name=new Name();}

   return name.getQName();
}

/**
* Sets the Qualified Name (QName) for the component.  Overrides current local name and prefix, if any.
**/
public void setQName(String QName){
   if (name==null) {name=new Name();}
   name.setQName(QName);
}

/**
* Returns the name object for the component.
**/
public NamingI getName(){
  return name;
}

/**
* Sets the name object for the component. 
**/
public void setName(NamingI _name){
   name=(Name) _name;
}


public CoreComponentI getParent(){
    return parent;
}

public void setParent(CoreComponentI _parent){
    parent=_parent;
}


public void setContent(String _content){
    content=new ComponentList();
    Characters thisText=new Characters();
    thisText.setContent(_content);
    content.add(thisText);
}

public void setContent(ComponentListI _content){
   content=(ComponentList) _content;
}

public void addContent(CoreComponentI newComponent){
   if (content==null) {
     content=new ComponentList();
   }
   content.add(newComponent);
}

public void clearContent(){
   content=null;
}

public String getTextContent(){
   String returnVal;
   if (content==null) {
     returnVal="";
   } else {
     returnVal=content.getTextContent();
   }
   return returnVal;
}

public ComponentListI getContent(){
   return content;
}

public void addUnorderedContent(CoreComponentI newComponent){
   if (unorderedContent==null) {
     unorderedContent=new ComponentSet();
   }
   unorderedContent.add(newComponent);
}

public void clearUnorderedContent() {
   unorderedContent=null;
}

public ComponentSetI getUnorderedContent(){
   return unorderedContent;
}

public void setUnorderedContent(ComponentSetI newContent){
   unorderedContent=(ComponentSet) newContent;
}

public void clearName(){
   name=null;
}

public void clearAll(){
  name=null;
  content=null;
  unorderedContent=null;
  complete=false;
  annotations=new Annotations();

}


public void print() {
   System.out.println(this.toString());
}

public abstract String toString();

public void setIsComplete(boolean _complete){
   complete=_complete;
}

public boolean getIsComplete(){
   return complete;
}
/**
* CoreComponent has no basic type - subclasses MUST provide one.
**/
public abstract int getBasicType();

public boolean listen (CoreComponentI component) throws MOEException{
    context=(CoreComponent)context.add(component, context);
    return this.getIsComplete();
}

/**
* Non-clone implementation needs replacement.  Will be made abstract when subclasses have implementation.
**/
public Object clone(){
   return (Object) this;
}

public void annotate(Object annotationType, Object annotation){
  annotations.annotate(annotationType, annotation);
}

public void addAll(Map otherAnnotations){
   annotations.addAll(otherAnnotations);
}
public Object getAnnotations(Object ofAnnotationType){
  return annotations.getAnnotations(ofAnnotationType);
}

public Map getAllAnnotations(){
  return annotations.getAllAnnotations();
}

public boolean hasAnnotationOfType(Object ofAnnotationType){
  return annotations.hasAnnotationOfType(ofAnnotationType);
}

public boolean hasAnnotation(Object annotation){
   return annotations.hasAnnotation(annotation);
}

public boolean hasAnnotations(){
   return annotations.hasAnnotations();
}

public Set getTypes(){
   return annotations.getTypes();
}

public void removeAnnotation(Object ofAnnotationType){
  annotations.removeAnnotation(ofAnnotationType);
}

public void removeAllAnnotations(){
  annotations.removeAllAnnotations();
}

protected CoreComponentI add(CoreComponentI component, CoreComponentI context) throws MOEException{
  CoreComponentI newContext=null;
/*
System.out.println("Incoming type:" + component.getBasicType());
System.out.println("Incoming content:" + component.toString());
System.out.println("Current context:" + context.toString());
*/

  switch (component.getBasicType()) {
 
     case ELEMENT:
        component.setParent(context);
        context.addContent(component);
        if (!(component.getIsComplete()) ) {
          newContext=component;
        } else {
          newContext=context;
        }
     break;

     case END_ELEMENT:
       if  (context.getBasicType()!=ELEMENT)  {
         throw new MOEException("Tried to end a non-element with an element end!");
       }

       if (! (context.getName().equals(component.getName())) ) {
         throw new MOEException("Tried to end "+context.getName()+" with "+component.getName()+"!");
       }
        context.setIsComplete(true);
        newContext=context.getParent();

     break;

     case ATTRIBUTE:
        component.setParent(context);
        context.addUnorderedContent(component);
        if (!(component.getIsComplete()) ) {
          newContext=component;
        } else {
          newContext=context;
        }
     break;

     case END_ATTRIBUTE:
       if  (context.getBasicType()!=ATTRIBUTE)  {
         throw new MOEException("Tried to end a non-attribute with an attribute end!");
       }

       if (! (context.getName().equals(component.getName())) ) {
         throw new MOEException("Tried to end "+context.getName()+" with "+component.getName()+"!");
       }
        context.setIsComplete(true);
        newContext=context.getParent();
     break;

     case NAMESPACE_DECL:
        component.setParent(context);
        context.addUnorderedContent(component);
        newContext=component;
     break;

     case END_NAMESPACE:
//er, uh, nothing for now.
     break;

     case COMMENT:
        component.setParent(context);
        context.addContent(component);
        newContext=component;
     break;

     case END_COMMENT:
       if  (context.getBasicType()!=COMMENT)  {
         throw new MOEException("Tried to end a non-comment with a comment end!");
       }

        context.setIsComplete(true);
        newContext=context.getParent();
     break;

     case CDATA:
        component.setParent(context);
        context.addContent(component);
        if (!(component.getIsComplete()) ) {
          newContext=component;
        } else {
          newContext=context;
        }
     break;

     case END_CDATA:
       if  (context.getBasicType()!=CDATA)  {
         throw new MOEException("Tried to end a non-CDATA-section with a CDATA end!");
       }

        context.setIsComplete(true);
        newContext=context.getParent();
     break;

     case PI:
        component.setParent(context);
        context.addContent(component);
        if (!(component.getIsComplete()) ) {
          newContext=component;
        } else {
          newContext=context;
        }
     break;

     case END_PI:
       if  (context.getBasicType()!=PI)  {
         throw new MOEException("Tried to end a non-PI with a PI end!");
       }
        context.setIsComplete(true);
        newContext=context.getParent();
     break;

     case CHARS:
        component.setParent(context);
        context.addContent(component);
        newContext=context;
     break;

     case IG_WHITESPACE:
        component.setParent(context);
        context.addContent(component);
        newContext=context;
     break;

  }
/*
if (newContext!=null) {
System.out.println("Outgoing context: "+newContext.toString());
} else {
System.out.println("Outgoing context: null");
}
*/
  return newContext;
}

}