package com.simonstl.moe;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The CoreComponent interface defines the bare minimum of functionality needed for objects to work with MOE.</p>
*
* <p>version 0.01 is the initial release.</p>
*
* @version 0.01 24 August 2001
* @author Simon St.Laurent
**/

public interface CoreComponentI extends NamingI, AnnotableI,Cloneable {

public static final int DOCUMENT=0;
public static final int END_DOCUMENT=1;
public static final int ELEMENT=2;
public static final int END_ELEMENT=3;
public static final int ATTRIBUTE=4;
public static final int END_ATTRIBUTE=5;
public static final int COMMENT=6;
public static final int END_COMMENT=7;
public static final int PI=8;
public static final int END_PI=9;
public static final int DOCTYPE=10;
public static final int END_DOCTYPE=11;
public static final int CDATA=12;
public static final int END_CDATA=13;

public static final int XML_DECL=18;
public static final int NAMESPACE_DECL=20;
public static final int END_NAMESPACE=21;
//need to consider DTD contents

public static final int GENERIC=64;
public static final int END_GENERIC=65;

public static final int GEN_ENTITY_START=130;
public static final int GEN_ENTITY_END=131;

public static final int GEN_ENTITY_REF=250;
public static final int IG_WHITESPACE=252;
public static final int CHARS=254;

public static final int UNPARSED=258;





/**
* Returns a key which can be used as a unique identifier for this component.
**/
public int hashCode();

/**
* Accepts a URI reference value which is used as the namespace for the component.
**/
public void setNsURI(String newNsURI) ;

/**
* Returns the URI reference value for the namespace.
**/
public String getNsURI() ;

/**
* Accepts a string which will serve as the local name for this component.
**/
public void setLocalName(String newLocalName);

/**
* Returns the local name for the component
**/
public String getLocalName();

/**
* Accepts a string which serves as the prefix for this component.
**/
public void setPrefix(String newPrefix);

/**
* Returns the prefix used to represent the namespace URI reference for this component.
**/
public String getPrefix();

/**
* Returns the Qualified Name (QName) for the component.
**/
public String getQName();

/**
* Sets the Qualified Name (QName) for the component.  Overrides current local name and prefix, if any.
**/
public void setQName(String QName);

/**
* Returns the name object for the component.
**/
public NamingI getName();

/**
* Sets the name object for the component. 
**/
public void setName(NamingI Name);


/**
* Returns the parent for the component.
**/
public CoreComponentI getParent();

/**
* Sets the parent object for the component.  Permits upward traversal of document trees.
**/
public void setParent(CoreComponentI _parent);


/**
* Sets the content for this component as a string.
**/
public void setContent(String newContent);

/**
* Sets the content for this component as a list of nodes.
**/
public void setContent(ComponentListI newContent);

/**
* Adds content to this component.
**/
public void addContent(CoreComponentI newComponent);

/**
* Clears the content for this component.
**/
public void clearContent();

/**
* Returns the content for this component as a string.
**/
public String getTextContent();

/**
* Returns the content for this component as a list of nodes.
**/
public ComponentListI getContent();



/**
* Adds an unordered component (usually an attribute) to a component.
**/
public void addUnorderedContent(CoreComponentI newComponent);

/**
* Clears the unordered content (usually attributes) for this component.
**/
public void clearUnorderedContent();

/**
* Returns the unordered content for this component.
**/
public ComponentSetI getUnorderedContent();

/**
* Sets the unordered content for this component.
**/
public void setUnorderedContent(ComponentSetI newContent);

/**
* Clears the name and namespace information for this component.
**/
public void clearName();

/**
* Clears the name, namespace information, and content for this component.
**/
public void clearAll();


/**
* Prints a representation of the component to STDOUT
**/
public void print() ;

/**
* Returns a representation of the component as a String. <strong>Note:</strong> toString() is really only guaranteed effective for debugging, because namespaces may require the additional intelligence of a visitor class.
**/
public String toString() ;

/**
* Returns a representation of the component as a String. Given a namespaceContext, this may be more effective than the regular toString().
**/
public String toString(ComponentSetI namespaceContext) ;



/**
* Sets whether this object is complete (i.e., an element has ended).
**/
public void setIsComplete(boolean complete);

/**
* Returns whether this object is complete (i.e., an element has ended).
**/
public boolean getIsComplete();

/**
* Returns the basic type of this object, per the constants defined in this interface.
<pre>
public static final int DOCUMENT=0;
public static final int END_DOCUMENT=1;
public static final int ELEMENT=2;
public static final int END_ELEMENT=3;
public static final int ATTRIBUTE=4;
public static final int END_ATTRIBUTE=5;
public static final int COMMENT=6;
public static final int END_COMMENT=7;
public static final int PI=8;
public static final int END_PI=9;
public static final int DOCTYPE=10;
public static final int END_DOCTYPE=11;
public static final int CDATA=12;
public static final int END_CDATA=13;


public static final int XML_DECL=18;
public static final int NAMESPACE_DECL=20;
public static final int END_NAMESPACE=21;

public static final int GENERIC=64;
public static final int END_GENERIC=65;

public static final int GEN_ENTITY_START=130;
public static final int GEN_ENTITY_END=131;

public static final int GEN_ENTITY_REF=250;
public static final int IG_WHITESPACE=252;
public static final int CHARS=254;

public static final int UNPARSED=258;</pre>


While classof tests are useful, there may be many different implementations of each of the types, likely with different object hierarchies.  This provides a simple form of self-identification.
**/
public int getBasicType();




/**
* Accepts new events.  Returns false until one of those events signals the completion of the current listener.  The listen() method is designed for 'conventional' XML use.  Attributes will be treated as unordered content, while everything else will be treated as ordered content.  If you need to break those expectations, you'll need to use setContent() or setUnorderedContent() directly.
**/
public boolean listen (CoreComponentI component) throws MOEException;


/**
* Returns a clean copy of the component and its contents.
**/
public abstract Object clone() ;


}