package com.simonstl.moe;

import java.io.PrintStream;
import java.io.PrintWriter;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s):   
*/

/**
* <p>The MOEException class is a very light rewriting of the public domain org.xml.SAXException class, though more changes will appear over time.</p>
*
* 
* <p>version 0.01 is the initial release and contains plenty of bogus placeholder functionality.</p>
*
* @version 0.01 26 August 2001
* @author Simon St.Laurent
**/


public class MOEException extends Exception {
    
    
    /**
     * Create a new MOEException.
     *
     * @param message The error or warning message.
     * 
     */
    public MOEException (String message) {
	super(message);
	this.exception = null;
    }
    
    
    /**
     * Create a new MOEException wrapping an existing exception.
     *
     * <p>The existing exception will be embedded in the new
     * one, and its message will become the default message for
     * the MOEException.</p>
     *
     * @param e The exception to be wrapped in a MOEException.
     */
    public MOEException (Exception e)
    {
	super();
	this.exception = e;
    }
    
    
    /**
     * Create a new MOEException from an existing exception.
     *
     * <p>The existing exception will be embedded in the new
     * one, but the new exception will have its own message.</p>
     *
     * @param message The detail message.
     * @param e The exception to be wrapped in a MOEException.
     * 
     */
    public MOEException (String message, Exception e)
    {
      super(message + " Reason " + e.getMessage());
	this.exception = e;
    }
    
    
    /**
     * Return a detail message for this exception.
     *
     * <p>If there is an embedded exception, and if the MOEException
     * has no detail message of its own, this method will return
     * the detail message from the embedded exception.</p>
     *
     * @return The error or warning message.
     * 
     */
    public String getMessage ()
    {
	String message = super.getMessage();
	
	if (message == null && exception != null) {
	    return exception.getMessage();
	} else {
	    return message;
	}
    }
    
    
    /**
     * Return the embedded exception, if any.
     *
     * @return The embedded exception, or null if there is none.
     */
    public Exception getException ()
    {
	return exception;
    }


    /**
     * Override toString to pick up any embedded exception.
     *
     * @return A string representation of this exception.
     */
    public String toString ()
    {
	if (exception != null) {
	    return exception.toString();
	} else {
	    return super.toString();
	}
    }
    
//thanks to Leigh Dodds for next three methods
    /**
    * Overrides Throwable.printStackTrace to recursively display stack trace
    * of nested exceptions
    */
   public void printStackTrace( PrintWriter writer )
   {
      super.printStackTrace( writer );
      if (exception != null)
      {
	exception.printStackTrace( writer );
      }
      writer.flush();
   }

   /**
    * Overrides Throwable.printStackTrace to recursively display stack trace
    * of nested exceptions
    */
   public void printStackTrace()
   {
      printStackTrace( System.err );
   }

   /**
    * Overrides Throwable.printStackTrace to recursively display stack trace
    * of nested exceptions
    */
   public void printStackTrace( PrintStream stream )
   {
      printStackTrace(new PrintWriter( stream ) );
      stream.flush();
   }


    
    //////////////////////////////////////////////////////////////////////
    // Internal state.
    //////////////////////////////////////////////////////////////////////


    /**
     * @serial The embedded exception if tunnelling, or null.
     */    
    private Exception exception;
    
}

// end of MOEException.java

