package com.simonstl.moe;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLFilterImpl;
import java.util.List;
import java.util.Map;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The Mutable class provides a foundation for generic MOE objects.  Mutable objects can store text, unordered content, or ordered content, and they can change personality at the flip of a setBasicType(int _type) switch.</p>
<p>The next step for Mutable is creating a toString() which supports calls to a visitor class.</p>
*
* 
* <p>version 0.01 is the initial release and contains plenty of bogus placeholder functionality.</p>
*
* @version 0.01 14 September 2001
* @author Simon St.Laurent
**/

public class Mutable extends CoreComponent implements MutableComponentI {

protected String textContent="";
protected int basicType=GENERIC;

public void setContent(String _content){
    content=null;
    textContent=_content;
}

public void addContent(CoreComponentI newComponent){
   if (content==null) {
     content=new ComponentList();
     if (!(textContent.equals(""))) {
       Mutable firstText=new Mutable();
       firstText.setBasicType(CHARS);
       firstText.setContent(textContent);
       content.add(firstText);
     }
   }
   content.add(newComponent);
}

public void clearContent(){
   content=null;
   textContent="";
}

public String getTextContent(){
   String returnVal;
   if (content==null) {
     returnVal=textContent;
   } else {
     returnVal=content.getTextContent();
   }

   return returnVal;
}

public ComponentListI getContent(){
   ComponentListI returnVal=content;
   if (textContent!=null) {
     returnVal=new ComponentList();
     if (!(textContent.equals(""))) {
       Mutable firstText=new Mutable();
       firstText.setContent(textContent);
       returnVal.add(firstText);
     }
   }
   return returnVal;
}

public void print() {
   System.out.println(this.toString());
}

/**
* Need to force configurable ref to visitor class
**/
public String toString() {
   String returnVal=new String();

   return returnVal;
}

/**
* Need to force configurable ref to visitor class
**/
public String toString(ComponentSetI namespaceContext) {
   String returnVal=new String();

   return returnVal;
}


/**
* The basic type for Mutable is whatever it's been set to be.
**/
public int getBasicType() {
   return basicType;
}

public void setBasicType(int _type) {
   basicType=_type;
}


/**
* For now, the Mutable class passes listening to its superclass.
**/
public boolean listen (CoreComponentI component) throws MOEException {
  return super.listen(component);
}



/**
* Clone implementation uses MutableComponentI, adds text if necessary.
**/
public  Object clone() {
   MutableComponentI newComponent=new Mutable();
   if (getName()!=null) {
     NamingI newName=(NamingI) this.getName().clone();
     newComponent.setName(newName);
   }
   if (getContent()!=null) {
     ComponentListI newContent=(ComponentListI) this.getContent().clone();
     newContent.setParent(newComponent);
     newComponent.setContent(newContent);
   }
   if (getUnorderedContent()!=null) {
     ComponentSetI newUnorderedContent=(ComponentSetI) this.getUnorderedContent().clone();
     newUnorderedContent.setParent(newComponent);
     newComponent.setUnorderedContent(newUnorderedContent);
   }
   if (getUnorderedContent()!=null) {
     AnnotableI thisAnnotations=(Annotations) this.getAllAnnotations();
     AnnotableI newAnnotations=(Annotations) thisAnnotations.clone();

     newComponent.addAll((Map) newAnnotations);
   }
   newComponent.setIsComplete(this.getIsComplete());
  if (content==null) {
    newComponent.setContent(this.getTextContent());
  }
  newComponent.setBasicType(this.getBasicType());
   return (Object) newComponent;
}


}