package com.simonstl.moe;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLFilterImpl;
import java.util.List;
import java.util.StringTokenizer;


/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The Name class implements the minimum of functionality needed to support XML names for MOE objects.</p>
*
* <p>version 0.01 is the initial release and contains plenty of bogus placeholder functionality.</p>
*
* @version 0.01 1 September 2001
* @author Simon St.Laurent
**/

public class Name implements NamingI {

/**
* contains the namespace URI for the component (if any)
**/
  protected String nsURI=null;
/**
* contains the local name for the component (if any)
**/
  protected String localName=null;
/**
* contains the namespace prefix for the component (if any).  prefix:localname yields the QName for the component.
**/
  protected String prefix=null;


/**
* The hashCode() method uses nsURI+">"+localName to generate an int.  If you need something more precise, you'll need to override this.
**/
public int hashCode(){
  String identifier=nsURI+">"+localName;
  return identifier.hashCode();
}

public void setNsURI(String _nsURI) {
   nsURI=_nsURI;
}

public String getNsURI() {
   return nsURI;
}

public void setLocalName(String _localName){
   localName=_localName;
}

public String getLocalName(){
   return localName;
}

public void setPrefix(String _prefix){
   prefix=_prefix;
}

public String getPrefix(){
   return prefix;
}

/**
* Returns the Qualified Name (QName) for the component.
**/
public String getQName(){
   String returnVal="";

   if ( (prefix!=null) && (!(prefix.equals(""))) ) {
     returnVal=prefix+":";
   }
   returnVal=returnVal+localName;

   return returnVal;
}

/**
* Sets the Qualified Name (QName) for the component.  Overrides current local name and prefix, if any.
**/
public void setQName(String QName){
  String[] parts=breakQName(QName);
  prefix=parts[0];
  localName=parts[1];
}

public void print() {
   System.out.println(this.toString());
}

public String toString(){ 
  return nsURI+">"+localName;

}

public boolean equals(Object _name) {
  return ( (_name.toString()).equals(this.toString()) ) ;
}

/**
Divides a QName into two pieces.
**/
public static String[] breakQName(String QName){
   String[] returnVal=new String[2];
   StringTokenizer parts=new StringTokenizer(QName,":");
   if (parts.countTokens()==1){
     returnVal[0]=null;
     returnVal[1]=QName;
   }
   if (parts.countTokens()==2){
     returnVal[0]=parts.nextToken();
     returnVal[1]=parts.nextToken();
   }
  return returnVal;
}


/**
* Clones the name.
**/
public  Object clone() {
  Name newName=new Name();
  if (prefix!=null) {
    newName.setPrefix(new String(prefix));
  }
  if (localName!=null) {
    newName.setLocalName(new String(localName));
  }
  if (nsURI!=null) {
    newName.setNsURI(new String(nsURI));
  }
  return (Object) newName;
}

}