package com.simonstl.moe;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLFilterImpl;



/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The NamespaceName class provides a foundation for MOE objects to store namespaces.</p>
*
* <p>I've defined the Namespace class to be something of a broken name, much like 
* Namespace declarations in XML 1.0.  The prefix of the attribute is fixed to xmlns, 
* the localName defines the prefix defined by the namespace.  The toString() method
* handles the xmlns/xmlns:prefix issues.</p>
** <p>All of the processing of the  {@link com.simonstl.moe.NamingI NamingI} interface is delegated to a {@link com.simonstl.moe.NamespaceName NamespaceName} object.</p>

* <p>nsURI is fixed to <code>http://www.w3.org/TR/REC-xml-names/</code>.</p>
*

* <p>version 0.01 is the initial release and contains plenty of bogus placeholder functionality.</p>
*
* @version 0.01 1 September 2001
* @author Simon St.Laurent
**/

public class NamespaceName implements NamingI {

/**
* contains the prefix used by the namespace (if any)
**/
  protected String localName=null;

/**
* The hashCode() method uses xmlns(:prefix) to generate an int.  If you need something more precise, you'll need to override this.
**/
public int hashCode(){
  String identifier=this.toString();
  return identifier.hashCode();
}


/** Prefix for namespaces is always xmlns.
**/

public String getPrefix() {
   return "xmlns";
}
/** Attempts to set prefixes on namespace names will be ignored.
**/

public void setPrefix(String prefix){

}

public String getNsURI() {
   return "http://www.w3.org/TR/REC-xml-names/";
}

public void setNsURI(String nsURI){

}

public String getLocalName() {
   return localName;
}
/**
Use setLocalName to identify the namespace prefix being used.
**/
public void setLocalName(String _localName){
   localName=_localName;
}


/**
* Returns the Qualified Name (QName) for the component. Overrides the usual
* QName approach because of the different approach used by Namespaces in XML.
**/
public String getQName(){
   String returnVal="xmlns";

   if (localName!=null) {
    returnVal=returnVal+":"+localName;
   }
   return returnVal;
}

/**
* Sets the Qualified Name (QName) for the component. Overrides the usual
* QName approach because of the different approach used by Namespaces in XML.
**/
public void setQName(String QName){
   String[] parts=Name.breakQName(QName);
   
   if (parts[0]==null) {
     localName=null;
   } else {
     localName=parts[1];
   }
}


public String toString() {
   String returnVal=this.getQName();
   return returnVal;
}

public boolean equals(Object _name) {
  return ( (_name.toString()).equals(this.toString()) ) ;
}

/**
* Clones the name.
**/
public  Object clone() {
  NamespaceName newName=new NamespaceName();
  newName.setLocalName(new String(localName));
  return (Object) newName;
}



}