package com.simonstl.moe.adapter;

import com.simonstl.moe.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.ArrayList;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The MOENodeWrapperA adapter class provides a view of individual MOE objects which will fit into a {@link com.simonstl.moe.adapter.MOETreeModelA MOETreeModelA} presentation.</p>
*
*<p>The MOENodeWrapperA class combines the unordered and ordered content into one big ordered collection for display. The toString() method returns the text content (for TEXT and IG_WHITESPACE) or the QName of the object (if it's appropriate) or &lt;-- --&gt;for a comment.</p>
*
* <p>version 0.01 is the initial release.</p>
*
* @version 0.01 20 September 2001
* @author Simon St.Laurent
**/

public class MOENodeWrapperA {

protected CoreComponentI component;
protected ArrayList children=new ArrayList();
protected boolean expanded=false;

public void setComponent(CoreComponentI _component) {
   component=_component;
//   component.print();
   if (component.getUnorderedContent()!=null){
     children.addAll(component.getUnorderedContent());
   }
   if (component.getContent()!=null){
   children.addAll(component.getContent());
   }
}

public CoreComponentI getComponent(){
   return component;
}

public String toString(){
   String returnVal=component.toString();

  if (expanded){

     if ( (component.getBasicType()==CoreComponentI.CHARS) || (component.getBasicType()==CoreComponentI.IG_WHITESPACE)) {
       returnVal=component.getTextContent();
     } else if (component.getBasicType()==CoreComponentI.COMMENT) {
       returnVal="<!-- -->";
     } else if (component.getBasicType()==CoreComponentI.CDATA) {
       returnVal="<[CDATA[ ]]>";
     } else {
       returnVal=component.getQName();
     }
   }
   
   return returnVal;
}

public String getToolTip() {
  return component.getNsURI();
}

public boolean isLeaf(){
   boolean returnVal=false;
   if ( (component.getBasicType()==CoreComponentI.CHARS) || (component.getBasicType()==CoreComponentI.IG_WHITESPACE)) {
     returnVal=true;
   }
   if (children.size()==0) {
     returnVal=true;
   }
   return returnVal;
}

public boolean getExpanded() {
   return expanded;
}

public void setExpanded(boolean _expanded){
   expanded=_expanded;
}

public Object getChild(int index) {
   MOENodeWrapperA returnVal=new MOENodeWrapperA();
   CoreComponentI child=(CoreComponentI) children.get(index);
   returnVal.setComponent(child);

   return returnVal;
}

public int getChildCount(){
return children.size();

}

public int getIndexOfChild(Object child){
return children.indexOf(child);

}

}