package com.simonstl.moe.explicit;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLFilterImpl;
import java.util.List;
import java.util.Map;
import com.simonstl.moe.*;

//test update of CVS
/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The CDATAsect class provides a foundation for MOE objects to store CDATA sections.(CDATA Sections are not reported by SAX parsers.)</p>
*
* 
* <p>version 0.01 is the initial release and contains plenty of bogus placeholder functionality.</p>
*
* @version 0.01 25 August 2001
* @author Simon St.Laurent
**/

public class CDATASect extends CoreComponent {


/**
* The hashCode() method is the same as Characters, but as CDATA nodes generally have no nsURI or localName they'll all collide if stored in a set.  CDATA sections must be stored in ordered content to work reliably.  If you really want to store CDATA in unordered content, override this.
**/
public int hashCode(){
  String identifier=getNsURI()+">"+getLocalName();
  return identifier.hashCode();
}

public void print() {
   System.out.println(this.toString());
}

public String toString() {
   String returnVal="<![CDATA[";
   returnVal=returnVal+this.getTextContent();
   if (complete) {
     returnVal=returnVal+"]]>";
   }
   return returnVal;
}

public String toString(ComponentSetI namespaceContext) {
   return this.toString();
}


/**
* CDATASect objects have basic type CDATA.
**/
public int getBasicType() {
   return CDATA;
}

/**
* For now, the CDATAsect class doesn't listen.  I may revisit this decison later.
**/
public boolean listen (CoreComponentI component) throws MOEException{
  return super.listen(component);
}



/**
* Clone implementation uses CoreComponent, adds text if necessary.
**/
public Object clone() {
   CoreComponentI newComponent=new CDATASect();
   if (getName()!=null) {
     NamingI newName=(NamingI) this.getName().clone();
     newComponent.setName(newName);
   }
   if (getContent()!=null) {
     ComponentListI newContent=(ComponentListI) this.getContent().clone();
     newContent.setParent(newComponent);
     newComponent.setContent(newContent);
   }
   if (getUnorderedContent()!=null) {
     ComponentSetI newUnorderedContent=(ComponentSetI) this.getUnorderedContent().clone();
     newUnorderedContent.setParent(newComponent);
     newComponent.setUnorderedContent(newUnorderedContent);
   }
   if (getUnorderedContent()!=null) {
     AnnotableI thisAnnotations=(Annotations) this.getAllAnnotations();
     AnnotableI newAnnotations=(Annotations) thisAnnotations.clone();

     newComponent.addAll((Map) newAnnotations);
   }
   newComponent.setIsComplete(this.getIsComplete());

   return (Object) newComponent;
}


public static void main (String[] args) throws Exception {
    int numArgs=args.length;
    CDATASect chars=new CDATASect();

 if (numArgs==0) {
    System.out.println("need arguments!");
 } else { 
 for (int i=0; i<numArgs; i++) {
   Characters addChars=new Characters();
   addChars.setContent(" "+args[i]);
   chars.addContent(addChars);
   chars.print();
   chars.setIsComplete(true);
   chars.print();
   chars.setIsComplete(false);

 }

 }

}

}