package com.simonstl.moe.explicit;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLFilterImpl;
import java.util.Map;
import com.simonstl.moe.*;


/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The Namespace class provides a foundation for MOE objects to store namespaces.</p>
*
* <p>There are some real complications in dealing with namespaces in an event context, 
* made only more difficult by the potential creation of partial trees.
* The SAX event startPrefixMapping() takes place before the startElement() call,
* and similarly endPrefixMapping() is reported after endElement().</p>
*
* <p>SAX also provides options for reporting/not reporting the attributes which 
* define namespaces.  In MOE, when events are assembled, the namespaces will be 
* associated with the elements containing the definitions.</p>
*
* <p>I've defined the Namespace class to be something of a broken attribute, much like 
* Namespace declarations in XML 1.0.  The prefix of the attribute is fixed to xmlns, 
* the localName defines the prefix defined by the namespace.  The toString() method
* handles the xmlns/xmlns:prefix issues.</p>
*
* <p>The namespace URI is stored as textual content, just as with attributes.  This
* potentially supports some rather perverse things like the use of general entities
* in the attribute values used to define namespaces.</p>
*
* <p>nsURI is fixed to <code>http://www.w3.org/TR/REC-xml-names/</code>.</p>
*

* <p>version 0.01 is the initial release and contains plenty of bogus placeholder functionality.</p>
*
* @version 0.01 25 August 2001
* @author Simon St.Laurent
**/

public class EndNamespace extends Namespace {



/**
* EndNamespace has the basic type END_NAMESPACE.
**/
public int getBasicType() {
   return END_NAMESPACE;
}

public Object clone() {
   CoreComponentI newComponent=new EndNamespace();
   if (getName()!=null) {
     NamingI newName=(NamingI) this.getName().clone();
     newComponent.setName(newName);
   }
   if (getContent()!=null) {
     ComponentListI newContent=(ComponentListI) this.getContent().clone();
     newContent.setParent(newComponent);
     newComponent.setContent(newContent);
   }
   if (getUnorderedContent()!=null) {
     ComponentSetI newUnorderedContent=(ComponentSetI) this.getUnorderedContent().clone();
     newUnorderedContent.setParent(newComponent);
     newComponent.setUnorderedContent(newUnorderedContent);
   }
   if (getUnorderedContent()!=null) {
     AnnotableI thisAnnotations=(Annotations) this.getAllAnnotations();
     AnnotableI newAnnotations=(Annotations) thisAnnotations.clone();

     newComponent.addAll((Map) newAnnotations);
   }
   newComponent.setIsComplete(this.getIsComplete());

   return (Object) newComponent;
}


/**
* End classes don't listen.
**/
public boolean listen (CoreComponentI component){
  return false;
}


}