package com.simonstl.moe.explicit;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLFilterImpl;
import java.util.List;
import java.util.Map;
import com.simonstl.moe.*;


/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The ProcInst class provides a foundation for MOE objects to store processing instructions.</p>
* <p>The target should be stored as the first of two text nodes, though I'm reconsidering that.</p>
* 
* <p>version 0.01 is the initial release and contains plenty of bogus placeholder functionality.</p>
*
* @version 0.01 25 August 2001
* @author Simon St.Laurent
**/

public class ProcInst extends Characters {


/**
* The hashCode() method is the same as Characters, but as PI nodes generally have no nsURI they may collide if stored in a set.  Processing Instructions must be stored in ordered content to work reliably.  If you really want to store PIs in unordered content, override this.
**/
public int hashCode(){
  String identifier=getNsURI()+">"+getLocalName();
  return identifier.hashCode();
}

public void print() {
   System.out.println(this.toString());
}

public String toString() {
   String returnVal="<?";
   returnVal=returnVal+getLocalName()+" ";
   returnVal=returnVal+this.getTextContent();
   if (complete) {
     returnVal=returnVal+"?>";
   }
   return returnVal;
}

public String toString(ComponentSetI namespaceContext) {
   return this.toString();
}


/**
* Processing Instruction objects have basic type PI.
**/
public int getBasicType() {
   return PI;
}



/**
* For now, the ProcInst class doesn't listen.  I may revisit this decison later.
**/
public boolean listen (CoreComponentI component){
  return false;
}

/**
* Clone implementation uses CoreComponent, adds text if necessary.
**/
public Object clone() {
   CoreComponentI newComponent=new ProcInst();
   if (getName()!=null) {
     NamingI newName=(NamingI) this.getName().clone();
     newComponent.setName(newName);
   }
   if (getContent()!=null) {
     ComponentListI newContent=(ComponentListI) this.getContent().clone();
     newContent.setParent(newComponent);
     newComponent.setContent(newContent);
   }
   if (getUnorderedContent()!=null) {
     ComponentSetI newUnorderedContent=(ComponentSetI) this.getUnorderedContent().clone();
     newUnorderedContent.setParent(newComponent);
     newComponent.setUnorderedContent(newUnorderedContent);
   }
   if (getUnorderedContent()!=null) {
     AnnotableI thisAnnotations=(Annotations) this.getAllAnnotations();
     AnnotableI newAnnotations=(Annotations) thisAnnotations.clone();

     newComponent.addAll((Map) newAnnotations);
   }
   newComponent.setIsComplete(this.getIsComplete());

   return (Object) newComponent;
}


public static void main (String[] args) throws Exception {
    int numArgs=args.length;
    ProcInst chars=new ProcInst();

 if (numArgs<2) {
    System.out.println("need more arguments!");
 } else { 
 chars.setLocalName(args[0]);
 for (int i=1; i<numArgs; i++) {
   Characters addChars=new Characters();
   addChars.setContent(" "+args[i]);
   chars.addContent(addChars);
   chars.print();
   chars.setIsComplete(true);
   chars.print();
   chars.setIsComplete(false);

 }

 }

}

}