package com.simonstl.moe.factory;

import com.simonstl.moe.*;
import com.simonstl.moe.explicit.*;
import java.lang.ClassNotFoundException;
import java.lang.IllegalAccessException;
import java.lang.InstantiationException;
import java.lang.SecurityException;
import java.lang.ClassCastException;
import java.util.StringTokenizer;


/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The ComponentQFactory class provides a relatively simple mechanism for creating new MOE components, using namespace URI and QName instead of of namspace URI, local name, prefix.  I genuinely detest the use of QNames in many contexts, but seem folks seem to prefer them.</p>
*
* <p>The ComponentQFactory class uses the same System properties as {@link com.simonstl.moe.factory.ComponentFactory ComponentFactory}  - it's just a wrapper around ComponentFactory.</p>
*
* @version 0.01 1 September 2001
* @author Simon St.Laurent
*/



public class ComponentQFactory {


/**
Divides a QName into two pieces.
**/
public static String[] breakQName(String QName){
   String[] returnVal=new String[2];
   StringTokenizer parts=new StringTokenizer(QName,":");
   if (parts.countTokens()==1){
     returnVal[0]=null;
     returnVal[1]=QName;
   }
   if (parts.countTokens()==2){
     returnVal[0]=parts.nextToken();
     returnVal[1]=parts.nextToken();
   }
  return returnVal;
}


/**
Creates an empty element with a namespace URI, local name, and prefix.
**/
public static CoreComponentI element(String _nsURI, String _QName) throws MOEException {
   String[] breaker=breakQName(_QName);
   String _prefix=breaker[0];
   String _localName=breaker[1];
   CoreComponentI returnVal=ComponentFactory.element(_nsURI, _localName, _prefix);

   return returnVal;
}

/**
Creates a complete element with a namespace URI, local name, prefix, and a text node.
**/

public static CoreComponentI element(String _nsURI, String _QName, String _text) throws MOEException  {
   Element returnVal=(Element) element(_nsURI, _QName);
   returnVal.setContent(_text);
   returnVal.setIsComplete(true);
   return returnVal;
}

/**
Creates an element with a namespace URI, local name, prefix, and a text node, and allows the caller to specify whether or not the element is complete.
**/

public static CoreComponentI element(String _nsURI, String _QName, String _text, boolean _complete) throws MOEException  {
   Element returnVal=(Element) element(_nsURI, _QName, _text);
   returnVal.setIsComplete(_complete);
   return returnVal;
}

/**
Creates an empty attribute with a namespace URI, local name, and prefix.
**/
public static CoreComponentI attribute(String _nsURI, String _QName) throws MOEException  {
   String[] breaker=breakQName(_QName);
   String _prefix=breaker[0];
   String _localName=breaker[1];

   CoreComponentI returnVal=ComponentFactory.attribute(_nsURI, _localName,_prefix);
   return returnVal;
}

/**
Creates an attribute with a namespace URI, local name, prefix, and a text node.
**/
public static CoreComponentI attribute(String _nsURI, String _QName, String _text)  throws MOEException {
   Attribute returnVal=(Attribute) attribute(_nsURI, _QName);
   returnVal.setContent(_text);
   returnVal.setIsComplete(true);
   return returnVal;
}

/**
Creates an attribute with a namespace URI, local name, prefix, and a text node, and allows the caller to specify whether or not the element is complete.
**/

public static CoreComponentI attribute(String _nsURI, String _QName, String _text, boolean _complete)  throws MOEException {
   Attribute returnVal=(Attribute) attribute(_nsURI, _QName, _text);
   returnVal.setIsComplete(_complete);
   return returnVal;
}


/**
Creates a text node with textual content.
**/
public static CoreComponentI characters(String _text) throws MOEException {
    CoreComponentI returnVal=ComponentFactory.characters(_text);
    return returnVal;
}

/**
Creates a complete Processing Instruction with target and textual content.
**/
public CoreComponentI procInst(String _target, String _text) throws MOEException {
  CoreComponentI returnVal=ComponentFactory.procInst(_target,_text);
    return returnVal;
}

/**
Creates a Processing Instruction with target and textual content, allowing the code to specify whether or not it is complete.
**/
public CoreComponentI procInst(String _target, String _text, boolean _complete) throws MOEException {
    ProcInst returnVal=(ProcInst) procInst(_target, _text);
    returnVal.setIsComplete(_complete);
    return returnVal;
}

/**
Creates a complete Comment with textual content.
**/
public static CoreComponentI comment(String _text) throws MOEException {
  CoreComponentI returnVal=ComponentFactory.comment(_text);
  return returnVal;
}

/**
Creates a comment with textual content, allowing the code to specify whether or not it is complete.
**/
public static CoreComponentI comment(String _text, boolean _complete) throws MOEException {
    Comment returnVal=(Comment) comment(_text);
    returnVal.setIsComplete(_complete);
    return returnVal;
}



/**
Creates a complete CDATA section with textual content.
**/
public static CoreComponentI cdataSect(String _text) throws MOEException {
   CoreComponentI returnVal=ComponentFactory.cdataSect(_text);
   return returnVal;
}

/**
Creates a CDATA section with textual content, allowing the code to specify whether or not it is complete.
**/
public static CoreComponentI cdataSect(String _text, boolean _complete) throws MOEException {

    CDATASect returnVal=(CDATASect) cdataSect(_text);
    returnVal.setIsComplete(_complete);
    return returnVal;
}


/**
Creates a whitespace node with content.
**/
public static CoreComponentI whitespace(String _text) throws MOEException {
  CoreComponentI returnVal=ComponentFactory.whitespace(_text);
    return returnVal;
}

/**
Creates a namespace.
**/
public static CoreComponentI namespace(String _nsURI, String _prefix)  throws MOEException {
  CoreComponentI returnVal=ComponentFactory.namespace(_nsURI, _prefix);
   return returnVal;
}

/**
Creates a default.
**/
public static CoreComponentI namespace(String _nsURI)  throws MOEException {
   return namespace(_nsURI, null);
}

/**
Creates an end element with a namespace URI, local name, and prefix.
**/
public static CoreComponentI endElement(String _nsURI, String _QName)  throws MOEException {
   String[] breaker=breakQName(_QName);
   String _prefix=breaker[0];
   String _localName=breaker[1];
   CoreComponentI returnVal=ComponentFactory.endElement(_nsURI, _localName, _prefix);

   return returnVal;
}

/**
Creates an empty attribute with a namespace URI, local name, and prefix.
**/
public static CoreComponentI endAttribute(String _nsURI, String _QName)  throws MOEException {
   String[] breaker=breakQName(_QName);
   String _prefix=breaker[0];
   String _localName=breaker[1];
   CoreComponentI returnVal=ComponentFactory.endAttribute(_nsURI, _localName, _prefix);

   return returnVal;
}

/**
Creates an end to a namespace.
**/
public static CoreComponentI endNamespace(String _nsURI, String _prefix)  throws MOEException {
  CoreComponentI returnVal=ComponentFactory.endNamespace(_nsURI, _prefix);
   return returnVal;
}



/**
Creates an end to a CDATA section.
**/
public static CoreComponentI endCdataSect() throws MOEException {
  CoreComponentI returnVal=ComponentFactory.endCdataSect();
   return returnVal;
}

public static CoreComponentI endProcInst() throws MOEException {
  CoreComponentI returnVal=ComponentFactory.endProcInst();
   return returnVal;
}

public static CoreComponentI endComment() throws MOEException {
  CoreComponentI returnVal=ComponentFactory.endComment();
   return returnVal;
}


public static void main(String[] args) throws Exception {
  ComponentQFactory cf=new ComponentQFactory();
  System.out.println("Incomplete element, no text.");
  Element test1=(Element) cf.element("http://simonstl.com/ns/test", "prefix:test");
  test1.print();

  System.out.println("Incomplete element, no text.");
  Element test2=(Element) cf.element("http://simonstl.com/ns/test", "prefix:test", "textual content", false);
  test2.print();

  System.out.println("Complete element, no text.");
  Element test3=(Element) cf.element("http://simonstl.com/ns/test", "prefix:test", "textual content");
  test3.print();

  System.out.println("Attribute, no text.");
  Attribute test4=(Attribute) cf.attribute("http://simonstl.com/ns/test", "prefix:testAtt1");
  test4.print();

  System.out.println("Attribute, with text.");
  Attribute test5=(Attribute) cf.attribute("http://simonstl.com/ns/test", "prefix:testAtt2", "texties");
  test5.print();

  System.out.println("Attributes added to complete element.");
  test3.addUnorderedContent(test4);
  test3.addUnorderedContent(test5);
  test3.print();

  System.out.println("Attributes added repeatedly to complete element.");
  test3.addUnorderedContent(test4);
  test3.addUnorderedContent(test5);
  test3.print();

  System.out.println("CDATA section with text, incomplete.");
  CDATASect test6=(CDATASect) cf.cdataSect("weird content, ><", false);
  test6.print();

  System.out.println("CDATA section with text, complete.");
  CDATASect test7=(CDATASect) cf.cdataSect("weird content, ><");
  test7.print();

  System.out.println("Characters with text");
  Characters test8=(Characters) cf.characters("some content");
  test8.print();

  System.out.println("Namespace with prefix");
  Namespace test9=(Namespace) cf.namespace("http://simonstl.com", "goon");
  test9.print();

  System.out.println("Namespace without prefix");
  Namespace test10=(Namespace) cf.namespace("http://simonstl.com");
  test10.print();

}
}