package com.simonstl.moe.factory;

import com.simonstl.moe.*;
import com.simonstl.moe.explicit.*;
import java.util.StringTokenizer;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The DefaultFactory class provides a relatively simple mechanism for creating new MOE components, hardwired to the components provided in the com.simonstl.moe package.  I've kept the constructors on the components deliberately minimal, preferring to use factory classes to create particular objects</p>
*
* @version 0.01 29 August 2001
* @author Simon St.Laurent
* @deprecated
*/



public class DefaultFactory {

/**
Divides a QName into two pieces.
**/
public static String[] breakQName(String QName){
   String[] returnVal=new String[2];
   StringTokenizer parts=new StringTokenizer(QName,":");
   if (parts.countTokens()==1){
     returnVal[0]=null;
     returnVal[1]=QName;
   }
   if (parts.countTokens()==2){
     returnVal[0]=parts.nextToken();
     returnVal[1]=parts.nextToken();
   }
  return returnVal;
}

/**
Creates an empty element with a namespace URI, local name, and prefix.
**/
public static CoreComponentI element(String _nsURI, String _localName, String _prefix) {
   Element returnVal=new Element();
   returnVal.setNsURI(_nsURI);
   returnVal.setLocalName(_localName);
   returnVal.setPrefix(_prefix);
   return returnVal;
}

/**
Creates a complete element with a namespace URI, local name, prefix, and a text node.
**/

public static CoreComponentI element(String _nsURI, String _localName, String _prefix, String _text) {
   Element returnVal=(Element) element(_nsURI, _localName, _prefix);
   returnVal.setContent(_text);
   returnVal.setIsComplete(true);
   return returnVal;
}

/**
Creates an element with a namespace URI, local name, prefix, and a text node, and allows the caller to specify whether or not the element is complete.
**/

public static CoreComponentI element(String _nsURI, String _localName, String _prefix, String _text, boolean _complete) {
   Element returnVal=(Element) element(_nsURI, _localName, _prefix, _text);
   returnVal.setIsComplete(_complete);
   return returnVal;
}

/**
Creates an empty attribute with a namespace URI, local name, and prefix.
**/
public static CoreComponentI attribute(String _nsURI, String _localName, String _prefix) {
   Attribute returnVal=new Attribute();
   returnVal.setNsURI(_nsURI);
   returnVal.setLocalName(_localName);
   returnVal.setPrefix(_prefix);
   return returnVal;
}

/**
Creates an attribute with a namespace URI, local name, prefix, and a text node.
**/
public static CoreComponentI attribute(String _nsURI, String _localName, String _prefix, String _text) {
   Attribute returnVal=(Attribute) attribute(_nsURI, _localName, _prefix);
   returnVal.setContent(_text);
   returnVal.setIsComplete(true);
   return returnVal;
}

/**
Creates an attribute with a namespace URI, local name, prefix, and a text node, and allows the caller to specify whether or not the element is complete.
**/

public static CoreComponentI attribute(String _nsURI, String _localName, String _prefix, String _text, boolean _complete) {
   Attribute returnVal=(Attribute) attribute(_nsURI, _localName, _prefix, _text);
   returnVal.setIsComplete(_complete);
   return returnVal;
}


/**
Creates a text node with textual content.
**/
public static CoreComponentI characters(String _text){
    Characters returnVal=new Characters();
    returnVal.setContent(_text);
    return returnVal;
}

/**
Creates a complete CDATA section with textual content.
**/
public static CoreComponentI cdataSect(String _text){
    CDATASect returnVal=new CDATASect();
    returnVal.setContent(_text);
    returnVal.setIsComplete(true);
    return returnVal;
}

/**
Creates a CDATA section with textual content, allowing the code to specify whether or not it is complete.
**/
public static CoreComponentI cdataSect(String _text, boolean _complete){
    CDATASect returnVal=(CDATASect) cdataSect(_text);
    returnVal.setIsComplete(_complete);
    return returnVal;
}

/**
Creates a complete Processing Instruction with target and textual content.
**/
public static CoreComponentI procInst(String _target, String _text){
    ProcInst returnVal=new ProcInst();
    returnVal.setLocalName(_target);
    returnVal.setContent(_text);
    returnVal.setIsComplete(true);
    return returnVal;
}

/**
Creates a Processing Instruction with target and textual content, allowing the code to specify whether or not it is complete.
**/
public static CoreComponentI procInst(String _target, String _text, boolean _complete){
    ProcInst returnVal=(ProcInst) procInst(_target, _text);
    returnVal.setIsComplete(_complete);
    return returnVal;
}

/**
Creates a complete Comment with textual content.
**/
public static CoreComponentI comment(String _text){
    Comment returnVal=new Comment();
    returnVal.setContent(_text);
    returnVal.setIsComplete(true);
    return returnVal;
}

/**
Creates a comment with textual content, allowing the code to specify whether or not it is complete.
**/
public static CoreComponentI comment(String _text, boolean _complete){
    Comment returnVal=(Comment) comment(_text);
    returnVal.setIsComplete(_complete);
    return returnVal;
}


/**
Creates a whitespace node with content.
**/
public static CoreComponentI whitespace(String _text){
    Whitespace returnVal=new Whitespace();
    returnVal.setContent(_text);
    return returnVal;
}

/**
Creates a namespace.
**/
public static CoreComponentI namespace(String _nsURI, String _prefix) {
   Namespace returnVal=new Namespace();
   returnVal.setLocalName(_prefix);
   returnVal.setContent(_nsURI);
   return returnVal;
}

/**
Creates a default namespace.
**/
public static CoreComponentI namespace(String _nsURI) {
   Namespace returnVal=new Namespace();
   returnVal.setLocalName(null);
   returnVal.setContent(_nsURI);
   return returnVal;
}

/**
Creates an end element with a namespace URI, local name, and prefix.
**/
public static CoreComponentI endElement(String _nsURI, String _localName, String _prefix) {
   EndElement returnVal=new EndElement();
   returnVal.setNsURI(_nsURI);
   returnVal.setLocalName(_localName);
   returnVal.setPrefix(_prefix);
   return returnVal;
}

/**
Creates an empty attribute with a namespace URI, local name, and prefix.
**/
public static CoreComponentI endAttribute(String _nsURI, String _localName, String _prefix) {
   EndAttribute returnVal=new EndAttribute();
   returnVal.setNsURI(_nsURI);
   returnVal.setLocalName(_localName);
   returnVal.setPrefix(_prefix);
   return returnVal;
}

/**
Creates an end to a namespace.
**/
public static CoreComponentI endNamespace(String _nsURI, String _prefix) {
   EndNamespace returnVal=new EndNamespace();
   returnVal.setLocalName(_prefix);
   returnVal.setContent(_nsURI);
   return returnVal;
}

/**
Creates an end to a CDATA section.
**/
public static CoreComponentI endCdataSect(){
   EndCDATASect returnVal=new EndCDATASect();
   return returnVal;
}

/**
Creates an end to a Processing Instruction.
**/
public static CoreComponentI endProcInst(){
   EndProcInst returnVal=new EndProcInst();
   return returnVal;
}

/**
Creates an end to a Comment.
**/
public static CoreComponentI endComment(){
   EndComment returnVal=new EndComment();
   return returnVal;
}


public static void main(String[] args) {
  System.out.println("Incomplete element, no text.");
  Element test1=(Element) element("http://simonstl.com/ns/test", "test", "prefix");
  test1.print();

  System.out.println("Incomplete element, no text.");
  Element test2=(Element) element("http://simonstl.com/ns/test", "test", "prefix", "textual content", false);
  test2.print();

  System.out.println("Complete element, no text.");
  Element test3=(Element) element("http://simonstl.com/ns/test", "test", "prefix", "textual content");
  test3.print();

  System.out.println("Attribute, no text.");
  Attribute test4=(Attribute) attribute("http://simonstl.com/ns/test", "testAtt1", "prefix");
  test4.print();

  System.out.println("Attribute, with text.");
  Attribute test5=(Attribute) attribute("http://simonstl.com/ns/test", "testAtt2", "prefix", "texties");
  test5.print();

  System.out.println("Attributes added to complete element.");
  test3.addUnorderedContent(test4);
  test3.addUnorderedContent(test5);
  test3.print();

  System.out.println("Attributes added repeatedly to complete element.");
  test3.addUnorderedContent(test4);
  test3.addUnorderedContent(test5);
  test3.print();

  System.out.println("CDATA section with text, incomplete.");
  CDATASect test6=(CDATASect) cdataSect("weird content, ><", false);
  test6.print();

  System.out.println("CDATA section with text, complete.");
  CDATASect test7=(CDATASect) cdataSect("weird content, ><");
  test7.print();

  System.out.println("Characters with text");
  Characters test8=(Characters) characters("some content");
  test8.print();

  System.out.println("Namespace with prefix");
  Namespace test9=(Namespace) namespace("http://simonstl.com", "goon");
  test9.print();

  System.out.println("Namespace without prefix");
  Namespace test10=(Namespace) namespace("http://simonstl.com");
  test10.print();

}

}