package com.simonstl.moe.factory;

import com.simonstl.moe.*;
import com.simonstl.moe.explicit.*;
import java.lang.ClassNotFoundException;
import java.lang.IllegalAccessException;
import java.lang.InstantiationException;
import java.lang.SecurityException;
import java.lang.ClassCastException;
import java.util.StringTokenizer;


/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The MutableFactory class provides a relatively simple mechanism for creating new MOE Mutable components.  I've kept the constructors on the Mutable class deliberately minimal, preferring to use factory classes to create particular objects.</p>
*
* @version 0.01 18 September 2001
* @author Simon St.Laurent
*/



public class MutableFactory {


/**
Creates an empty element with a namespace URI, local name, and prefix.
**/
public static MutableComponentI element(String _nsURI, String _localName, String _prefix) throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.ELEMENT);
   returnVal.setNsURI(_nsURI);
   returnVal.setLocalName(_localName);
   returnVal.setPrefix(_prefix);
   return returnVal;
}

/**
Creates a complete element with a namespace URI, local name, prefix, and a text node.
**/

public static MutableComponentI element(String _nsURI, String _localName, String _prefix, String _text) throws MOEException  {
   MutableComponentI returnVal=element(_nsURI, _localName, _prefix);
   returnVal.setContent(_text);
   returnVal.setIsComplete(true);
   return returnVal;
}

/**
Creates an element with a namespace URI, local name, prefix, and a text node, and allows the caller to specify whether or not the element is complete.
**/

public static MutableComponentI element(String _nsURI, String _localName, String _prefix, String _text, boolean _complete) throws MOEException  {
   MutableComponentI returnVal=element(_nsURI, _localName, _prefix, _text);
   returnVal.setIsComplete(_complete);
   return returnVal;
}

/**
Creates an empty attribute with a namespace URI, local name, and prefix.
**/
public static MutableComponentI attribute(String _nsURI, String _localName, String _prefix) throws MOEException  {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.ATTRIBUTE);
   returnVal.setNsURI(_nsURI);
   returnVal.setLocalName(_localName);
   returnVal.setPrefix(_prefix);
   return returnVal;
}

/**
Creates an attribute with a namespace URI, local name, prefix, and a text node.
**/
public static MutableComponentI attribute(String _nsURI, String _localName, String _prefix, String _text)  throws MOEException {
   MutableComponentI returnVal=attribute(_nsURI, _localName, _prefix);
   returnVal.setContent(_text);
   returnVal.setIsComplete(true);
   return returnVal;
}

/**
Creates an attribute with a namespace URI, local name, prefix, and a text node, and allows the caller to specify whether or not the element is complete.
**/

public static MutableComponentI attribute(String _nsURI, String _localName, String _prefix, String _text, boolean _complete)  throws MOEException {
   MutableComponentI returnVal=attribute(_nsURI, _localName, _prefix, _text);
   returnVal.setIsComplete(_complete);
   return returnVal;
}


/**
Creates a text node with textual content.
**/
public static MutableComponentI characters(String _text) throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.CHARS);
   returnVal.setContent(_text);
   return returnVal;
}

/**
Creates a complete Processing Instruction with target and textual content.
**/
public static MutableComponentI procInst(String _target, String _text) throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.PI);
   returnVal.setLocalName(_target);
   returnVal.setContent(_text);
   returnVal.setIsComplete(true);
   return returnVal;
}

/**
Creates a Processing Instruction with target and textual content, allowing the code to specify whether or not it is complete.
**/
public MutableComponentI procInst(String _target, String _text, boolean _complete) throws MOEException {
    MutableComponentI returnVal=procInst(_target, _text);
    returnVal.setIsComplete(_complete);
    return returnVal;
}

/**
Creates a complete Comment with textual content.
**/
public static MutableComponentI comment(String _text) throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.COMMENT);
    returnVal.setContent(_text);
    returnVal.setIsComplete(true);
    return returnVal;
}

/**
Creates a comment with textual content, allowing the code to specify whether or not it is complete.
**/
public static MutableComponentI comment(String _text, boolean _complete) throws MOEException {

    MutableComponentI returnVal=comment(_text);
    returnVal.setIsComplete(_complete);
    return returnVal;
}



/**
Creates a complete CDATA section with textual content.
**/
public static MutableComponentI cdataSect(String _text) throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.CDATA);
   returnVal.setContent(_text);
   returnVal.setIsComplete(true);
   return returnVal;
}

/**
Creates a CDATA section with textual content, allowing the code to specify whether or not it is complete.
**/
public static MutableComponentI cdataSect(String _text, boolean _complete) throws MOEException {

    MutableComponentI returnVal=cdataSect(_text);
    returnVal.setIsComplete(_complete);
    return returnVal;
}


/**
Creates a whitespace node with content.
**/
public static MutableComponentI whitespace(String _text) throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.IG_WHITESPACE);
    returnVal.setContent(_text);
    return returnVal;
}

/**
Creates a namespace.
**/
public static MutableComponentI namespace(String _nsURI, String _prefix)  throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.NAMESPACE_DECL);
   returnVal.setLocalName(_prefix);
   returnVal.setContent(_nsURI);
   return returnVal;
}

/**
Creates a default.
**/
public static MutableComponentI namespace(String _nsURI)  throws MOEException {
   return namespace(_nsURI, null);
}

/**
Creates an end element with a namespace URI, local name, and prefix.
**/
public static MutableComponentI endElement(String _nsURI, String _localName, String _prefix)  throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.END_ELEMENT);
   returnVal.setNsURI(_nsURI);
   returnVal.setLocalName(_localName);
   returnVal.setPrefix(_prefix);
   return returnVal;
}

/**
Creates an empty attribute with a namespace URI, local name, and prefix.
**/
public static MutableComponentI endAttribute(String _nsURI, String _localName, String _prefix)  throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.END_ATTRIBUTE);
   returnVal.setNsURI(_nsURI);
   returnVal.setLocalName(_localName);
   returnVal.setPrefix(_prefix);
   return returnVal;
}

/**
Creates an end to a namespace.
**/
public static MutableComponentI endNamespace(String _nsURI, String _prefix)  throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.END_NAMESPACE);
   returnVal.setLocalName(_prefix);
   returnVal.setContent(_nsURI);
   return returnVal;
}



/**
Creates an end to a CDATA section.
**/
public static MutableComponentI endCdataSect() throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.END_CDATA);
   return returnVal;
}

public static MutableComponentI endProcInst() throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.END_PI);
   return returnVal;
}

public static MutableComponentI endComment() throws MOEException {
   MutableComponentI returnVal;
   returnVal=new Mutable();
   returnVal.setBasicType(MutableComponentI.END_COMMENT);
   return returnVal;
}

/**
* Need to update for this factory.
**/
public static void main(String[] args) throws Exception {
  ComponentFactory cf=new ComponentFactory();
  System.out.println("Incomplete element, no text.");
  Element test1=(Element) cf.element("http://simonstl.com/ns/test", "test", "prefix");
  test1.print();

  System.out.println("Incomplete element, no text.");
  Element test2=(Element) cf.element("http://simonstl.com/ns/test", "test", "prefix", "textual content", false);
  test2.print();

  System.out.println("Complete element, no text.");
  Element test3=(Element) cf.element("http://simonstl.com/ns/test", "test", "prefix", "textual content");
  test3.print();

  System.out.println("Attribute, no text.");
  Attribute test4=(Attribute) cf.attribute("http://simonstl.com/ns/test", "testAtt1", "prefix");
  test4.print();

  System.out.println("Attribute, with text.");
  Attribute test5=(Attribute) cf.attribute("http://simonstl.com/ns/test", "testAtt2", "prefix", "texties");
  test5.print();

  System.out.println("Attributes added to complete element.");
  test3.addUnorderedContent(test4);
  test3.addUnorderedContent(test5);
  test3.print();

  System.out.println("Attributes added repeatedly to complete element.");
  test3.addUnorderedContent(test4);
  test3.addUnorderedContent(test5);
  test3.print();

  System.out.println("CDATA section with text, incomplete.");
  CDATASect test6=(CDATASect) cf.cdataSect("weird content, ><", false);
  test6.print();

  System.out.println("CDATA section with text, complete.");
  CDATASect test7=(CDATASect) cf.cdataSect("weird content, ><");
  test7.print();

  System.out.println("Characters with text");
  Characters test8=(Characters) cf.characters("some content");
  test8.print();

  System.out.println("Namespace with prefix");
  Namespace test9=(Namespace) cf.namespace("http://simonstl.com", "goon");
  test9.print();

  System.out.println("Namespace without prefix");
  Namespace test10=(Namespace) cf.namespace("http://simonstl.com");
  test10.print();

}
}