package com.simonstl.moe.factory;

import com.simonstl.moe.*;
import com.simonstl.moe.explicit.*;
import com.simonstl.moe.test.*;
import org.xml.sax.*;
import org.xml.sax.helpers.XMLFilterImpl;
import org.xml.sax.helpers.DefaultHandler;
import java.io.*;
import java.util.*;
import com.megginson.sax.XMLWriter;
import com.megginson.sax.DataWriter;
import org.xml.sax.helpers.ParserAdapter;
import org.xml.sax.helpers.XMLReaderFactory;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The SAXFactory class provides a relatively simple mechanism for creating new MOE components, hardwired to the components provided in the com.simonstl.moe package.  It in turn relies on the {@link com.simonstl.moe.factory.ComponentFactory ComponentFactory} class.</p>

<p>This class extends XMLFilterImpl rather than DefaultHandler so that it can be used as a pass-through if appropriate.</p>
*
* @version 0.01 29 August 2001
* @author Simon St.Laurent
*/



public class SAXFactory extends XMLFilterImpl implements ComponentOutputI{

protected int status=1;
protected ComponentInputI listener;
protected ComponentQFactory cqf=new ComponentQFactory();

public SAXFactory (XMLReader parent) {
    super(parent); 
} 


public int getStatus(){
  return status;
}

public boolean setReport(int typeReport){
  boolean returnVal=false;
  if (typeReport==SAX) {
    returnVal=true;
  }
  return returnVal;
}

/**
* Not sure I'll keep this.
**/
public CoreComponentI getCurrentComponent(){
  return null;
}

/**
It's all gotta go someplace.
**/

public void setListener(ComponentInputI _listener){
  listener=_listener;
}

public void startElement(String uri, String localname, String qname, Attributes atts) throws SAXException {
  CoreComponentI newAttribute=null;
  CoreComponentI newElement=null;

  try {
     newElement=cqf.element(uri, qname);
  } catch (MOEException e) {
    throw new SAXException(e);
  }

  //on to attributes
    int numAtts=atts.getLength();
    for (int i=0; i<numAtts; i++) {
     String attNsURI=atts.getURI(i);
     String attLocalName=atts.getLocalName(i);
     String attQName=atts.getQName(i);
     String content=atts.getValue(i);
     //type=atts.getType(i);
     try {
       newAttribute=cqf.attribute(attNsURI, attQName, content);
     } catch (MOEException e) {
       throw new SAXException(e);
     }
     newElement.addUnorderedContent(newAttribute);  
    } //end for

  try {
     if(listener!=null) {
       listener.listen(newElement);
    }
  } catch (MOEException e) {
    throw new SAXException(e);
  }


  super.startElement(uri, localname, qname, atts); 

}

public void endElement(String uri, String localname, String qname) throws SAXException {
  CoreComponentI newEndElement=null;

  try {
     newEndElement=cqf.endElement(uri, qname);
     if(listener!=null) {
       listener.listen(newEndElement);
    }
  } catch (MOEException e) {
    throw new SAXException(e);
  }
     super.endElement(uri, localname, qname);

}

public void characters(char[] ch, int offset, int len) throws SAXException{
  CoreComponentI newCharacters=null;

   String chString=new String (ch);
   String localString=chString.substring (offset, offset+len);
  try {
     newCharacters=cqf.characters(localString);
     if(listener!=null) {
       listener.listen(newCharacters);
    }
  } catch (MOEException e) {
    throw new SAXException(e);
  }
  super.characters (ch, offset, len);
}

public void ignorableWhitespace(char[] ch, int offset, int len) throws SAXException{
  CoreComponentI newWhitespace=null;

   String chString=new String (ch);
   String localString=chString.substring (offset, offset+len);
  try {
     newWhitespace=cqf.whitespace(localString);
     if(listener!=null) {
       listener.listen(newWhitespace);
     }
  } catch (MOEException e) {
    throw new SAXException(e);
  }
  super.ignorableWhitespace (ch, offset, len);

}


public void startPrefixMapping(String prefix, String uri) throws SAXException{
  CoreComponentI newNamespace=null;

  try {
     newNamespace=cqf.namespace(uri, prefix);
     if(listener!=null) {
      listener.listen(newNamespace);
     }
  } catch (MOEException e) {
    throw new SAXException(e);
  }
  super.startPrefixMapping(prefix, uri);

}

public void endPrefixMapping(String prefix) throws SAXException{
  CoreComponentI newEndNamespace=null;

  try {
     newEndNamespace=cqf.endNamespace("", prefix);
     if(listener!=null) {
       listener.listen(newEndNamespace);
     }
  } catch (MOEException e) {
    throw new SAXException(e);
  }
  super.endPrefixMapping(prefix);

}

/**
printHelp() lists options if the user doesn't specify any files.
*/

public static void printHelp() {
    System.out.println("FilterTester requires two arguments - a rules file and then a target XML file");
    System.out.println("");
    System.out.println("Options may appear before those arguments:");
    System.out.println("-p [parser class name] - specifies a SAX2 parser for use with this tool");
    System.out.println("More information is available at http://simonstl.com/projects/moe/");
    System.out.println("");
/*
    System.out.println("");
    System.out.println("");
    System.out.println("");
    System.out.println("");
*/
}

/**
FilterTester's main method requires two arguments - a rules file and then a target XML file.

Options may appear before those arguments:<br />
-p [parser wrapper class name] - specifies a SAX2 parser for use with this tool<br />
/>

*/

  // Main Method
  public static void main (String[] args) throws SAXException, IOException, FileNotFoundException {

    int numArgs=args.length;
    boolean printRules=false;
    String rulesFile;
    String targetFile;

//deal with System properties
   if ((System.getProperty("org.xml.sax.driver"))==null) {
      System.setProperty("org.xml.sax.driver","org.apache.xerces.parsers.SAXParser");
   }

   String parserName=System.getProperty("org.xml.sax.driver");

    if (numArgs<1) {
      printHelp();
    } else {
//process arguments
      targetFile=args[numArgs-1];
      for (int i=0; i<numArgs; i++) {
         if (args[i].equals ("-p")) {
           parserName=args[i+1];
           i++;
      }
    }

       //  Create a SAX Parser
   XMLReader rulesParser = XMLReaderFactory.createXMLReader(parserName); 

   XMLReader parser = XMLReaderFactory.createXMLReader(parserName); 

   ComponentInputPrint printer=new ComponentInputPrint();
   SAXFactory thisFilter=new SAXFactory(parser);
   thisFilter.setListener(printer);
 //  DataWriter w = new DataWriter(thisFilter);
   thisFilter.parse(new InputSource(new FileReader(targetFile)));
    }
  }

}