package com.simonstl.moe.namespace;

import com.simonstl.moe.*;
import com.simonstl.moe.explicit.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.HashSet;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.Attributes;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The NamespaceContext class masquerades as a {@link com.simonstl.moe.ComponentSet ComponentSet}, which actually storing
* namespace information in a stack.  This allows the preservation of namespaces which
* have been overridden so that they can reappear when the override context is finished.</p>
* <p>In many ways, this class is an illustration of the enormous nuisance Namespaces 
* in XML has created.</p>
*<p>This class is just getting started.  Most everything about it is wrong at present.</p>
*
* <p>Version 0.01 is Abstract class.</p>
*
* @version 0.01 24 August 2001
* @author Simon St.Laurent
*/
public class NamespaceContext extends ArrayList implements ComponentSetI, ComponentInputI {

/*
Make another class something that can be used for annotation, not something which rides the whole tree.  That should permit the use of HashSet, and local annotation rather than levels.

This class is overall an ArrayList, but the Iterator works over a HashSet, not the ArrayList.  The HashSet is built to include only current namespaces.  -YEP

Use a listen() method to pick up new namespaces coming through.

Might also build a listener which just checks balance of start/end 

Add parent links to CoreComponent so elements can figure out their context without annotation on every element.
*/

protected int level=0;
protected boolean modified=true;
protected NamespaceSetI context;

public boolean add(Object o) {
  boolean returnVal=false;

  if (o instanceof Namespace) {
    returnVal=super.add(o);
  }
  modified=true;
  return returnVal;
}

public boolean addAll(Collection c) {
  boolean returnVal=false;
//should I check that all objects are Namespace?  Sheesh.
  returnVal=super.add(c);
  modified=true;
 
  return returnVal;
}

public void clear() {
  super.clear();
  modified=true;
}

public boolean contains(Object o) {
  boolean returnVal=false;

  returnVal=super.contains(o);

  return returnVal;
}

public boolean containsAll(Collection c) {
  boolean returnVal=false;

  returnVal=super.containsAll(c);


  return returnVal;
}

public Iterator iterator() {
//critical!
  return getContext().iterator();
}

public NamespaceSetI getContext() {
 if (modified) {
   context=new NamespaceSet();
   Iterator it=super.iterator();

   while (it.hasNext()) {
      Namespace next=(Namespace) it.next();
      context.add(next);
   }
  modified=false;

 }
   return context;
}

public boolean remove(Object o) {
  boolean returnVal=false;

  returnVal=super.remove(o);
  modified=true;

  return returnVal;
}

public boolean removeAll(Collection c) {
  boolean returnVal=false;
  returnVal=super.removeAll(c);
  modified=true;

  return returnVal;
}

public boolean retainAll(Collection c) {
  boolean returnVal=false;
  returnVal=super.retainAll(c);
  modified=true;

  return returnVal;
}

public int size() {
  int returnVal=0;
  returnVal=super.size();
  return returnVal;
}

public Object[] toArray(){
  modified=true;

  return super.toArray();
}

public Object[] toArray(Object[] a){
  modified=true;

  return super.toArray(a);
}

public String toString(){
  return this.toAttributes();
}
public String toString(ComponentSet namespaceContext){
  return this.toAttributes(namespaceContext);
}


public String toAttributes(ComponentSet namespaceContext){
  return "ComponentSet.toAttributes() not yet implemented.";
}

public String toAttributes(){
   String returnVal="";
   
   Iterator it=this.iterator();

   while (it.hasNext()) {
      CoreComponentI next=(CoreComponentI) it.next();
//modify to support namespace elements
      if (next.getPrefix()!=null) {
        returnVal=returnVal+next.getPrefix()+":";
      }
      returnVal=returnVal+next.getLocalName()+"=\"";
      returnVal=returnVal+next.getTextContent()+"\"";

   }

   return returnVal;

}

public String toElements(ComponentSet namespaceContext){
  return "ComponentSet.toElements() not yet implemented.";
}

public String toElements(){
  return "ComponentSet.toElements() not yet implemented.";
}

public Namespace extractNamespace(CoreComponentI source) {
  Namespace returnVal=null;
    if (source.getBasicType()==CoreComponentI.ATTRIBUTE) {
      returnVal=new Namespace();
      String prefix=source.getPrefix();
      String localName=source.getLocalName();
      if (prefix.equals("xmlns")) {
        returnVal.setLocalName(localName);
        returnVal.setContent(source.getContent());
      } else if (localName.equals("xmlns")) {
        returnVal.setLocalName("");
        returnVal.setContent(source.getContent());
      }
    }
  return returnVal;
}

public boolean listen(CoreComponentI sound){
  int type=sound.getBasicType();
  int initialLevel=level;

  switch (type) {
    case CoreComponentI.ELEMENT:
      level++;
      ComponentSet thisAttributes=(ComponentSet) sound.getUnorderedContent();

      Iterator it=this.iterator();

      while (it.hasNext()) {
        CoreComponentI next=(CoreComponentI) it.next();
        if (next.getBasicType()==CoreComponentI.ATTRIBUTE) {
          Namespace asNamespace=this.extractNamespace(next);
          if (asNamespace!=null) {
            asNamespace.setLevel(level);
            this.add(asNamespace);
          }
        }
        if (next.getBasicType()==CoreComponentI.NAMESPACE_DECL) {
          Namespace asNamespace=(Namespace) next;
          asNamespace.setLevel(level);
          this.add(asNamespace);
        }
 
      }//end while


      if (sound.getIsComplete()) {level--;}
    break;

    case CoreComponentI.END_ELEMENT:
      level--;
    break;

    case CoreComponentI.NAMESPACE_DECL:
       Namespace thisNamespace=(Namespace) sound;
       thisNamespace.setLevel(level);
       this.add(thisNamespace);
    break;

    case CoreComponentI.END_NAMESPACE:
      //I don't think anything needs to happen here, actually.
      //I'll leave it for the sake of paranoia, though.
    break;

    case CoreComponentI.ATTRIBUTE:
          Namespace asNamespace=this.extractNamespace(sound);
          if (asNamespace!=null) {
            asNamespace.setLevel(level);
            this.add(asNamespace);
          }
    break;

  }

  if (level<initialLevel) {
    //strike namespaces of <current level.
  }

  return true;
}

public void setParent(CoreComponentI _parent){
   Iterator it=this.iterator();

   while (it.hasNext()) {
      CoreComponentI next=(CoreComponentI) it.next();
      next.setParent(_parent);
   }

}


}