package com.simonstl.moe.ui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.beans.*;
import com.simonstl.moe.adapter.*;
import com.simonstl.moe.*;
import com.simonstl.moe.test.*;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The MOEWorkshop class provides a Swing-based view of MOE objects.</p>
*
* <p>version 0.01 is the initial release.</p>
*
* @version 0.01 20 September 2001
* @author Simon St.Laurent
**/

public class MOEWorkshop {


public static void main (String[] args) {

  JFrame frame=new JFrame("MOE Workshop");

  frame.addWindowListener(new WindowAdapter()    {
     public void windowClosing(WindowEvent e) {
       System.exit(0);
     };
  });

  JMenuBar menuBar=new JMenuBar();
  frame.setJMenuBar(menuBar);

  JMenu fileMenu=createFileMenu();
  menuBar.add(fileMenu);

  JMenu viewMenu=createViewMenu();
  menuBar.add(viewMenu);


  JMenu lfMenu=createLfMenu(frame) ;
  menuBar.add(lfMenu);
/*
  JToolBar toolbar=new JToolBar();
  frame.getContentPane().add(toolbar, BorderLayout.NORTH);
*/

  CoreComponentI root=null;
  try {
    root=ComponentGenAttNest.generate();
  } catch (MOEException ex) {
    System.err.println(ex);
    System.exit(0);
  }

  MOETreeModelA treeModel=new MOETreeModelA();
  treeModel.setComponent(root);

  MOETree tree=new MOETree(treeModel);

  frame.getContentPane().add(new JScrollPane(tree));
  ToolTipManager.sharedInstance().registerComponent(tree);
  frame.pack();
  frame.setVisible(true);
}

public static JMenu createLfMenu(final JFrame frame) {
  JMenu lfMenu=new JMenu("Appearance");
  lfMenu.setMnemonic('A');

  ButtonGroup radioGroup=new ButtonGroup();

  UIManager.LookAndFeelInfo[] lfs=UIManager.getInstalledLookAndFeels();

  for (int i=0; i<lfs.length; i++) {
    String lfName=lfs[i].getName();
    final String lfClassName=lfs[i].getClassName();

    JMenuItem item=lfMenu.add(new JRadioButtonMenuItem(lfName));

    radioGroup.add(item);

    item.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
           UIManager.setLookAndFeel(lfClassName);

           SwingUtilities.updateComponentTreeUI(frame);

           frame.pack();

        } catch (Exception ex) {
           System.err.println(ex);
        }
      }
    });

  }
  
  return lfMenu;

}

public static JMenu createFileMenu() {
    JMenu fileMenu=new JMenu("File");
    fileMenu.setMnemonic('F');

    JMenuItem openItem=fileMenu.add(new JMenuItem("Open Pathway", 'O'));

    JMenuItem saveItem=fileMenu.add(new JMenuItem("Save Pathway", 'S'));
    fileMenu.addSeparator();

    JMenuItem runItem=fileMenu.add(new JMenuItem("Run Pathway", 'R'));

    fileMenu.addSeparator();
    JMenuItem exitItem=fileMenu.add(new JMenuItem("Exit", 'X'));


    exitItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          Quit();
        } catch (Exception ex) {
           System.err.println(ex);
        }
      }
    });

 
  
  return fileMenu;

}

public static JMenu createViewMenu() {
    JMenu viewMenu=new JMenu("View");
    viewMenu.setMnemonic('V');

    ButtonGroup radioGroup=new ButtonGroup();

    JMenuItem treeItem=viewMenu.add(new JRadioButtonMenuItem("Tree"));

    radioGroup.add(treeItem);

    JMenuItem rawItem=viewMenu.add(new JRadioButtonMenuItem("Raw XML"));

    radioGroup.add(rawItem);

    viewMenu.addSeparator();

  
  return viewMenu;

}


public static void Quit () throws Exception {
       System.exit(0);
}

}